from django.conf import settings
from .choices import ORIGIN_SERVER

import pghistory


class ConcordiaHistoryMiddleware:
    """
    Add User details to History context.
    Must be included AFTER middleware that
    adds the current user to the request!
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):

        user = request.user
        if (
                not user.is_authenticated
                or request.method not in settings.PGHISTORY_MIDDLEWARE_METHODS
                or request.path in settings.SKIP_MIDDLEWARE_URLS
        ):
            return self.get_response(request)

        with pghistory.context(**user_context(user)):
            return self.get_response(request)


def user_context(user) -> dict:
    """
    Return user context dictionary
    """
    return {
        'user': user.id if user else None,
        'user_cid': user.cid if user else None,
        'username': user.username if user else None,
        ORIGIN_SERVER.CID: settings.CONCORD_MQ_TENANT,
        ORIGIN_SERVER.Name: settings.SYSTEM_NAME,
    }
